/* DrumKey.c
 *
 * An example of using GENMIDI.DLL.
 *
 * This example shows how to call the DLL function MidiIsGMDrum to retrieve
 * the MIDI note number for a given General MIDI Drum Key name. When
 * running the program, supply a defined Drum Key Name. The program
 * will print the respective note number for that GM Drum Key name if a match is
 * found. For example, "Kick 1" is note #36. Otherwise, a message is printed
 * that the name is not found.
 */

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <string.h>
#include "..\genmidi.h"


/* ********************************* main() ********************************* */

void main(int argc, char *argv[])
{
	unsigned char * ptr;
	unsigned char   drum;

	/* Check if any parameters were given, if not, display some help info */
	if (argc < 2)
	{
		_cputs("Displays the note number for a specified General MIDI Drum Key Name.\r\n\nSyntax is '");
		_cputs(argv[0]);
		_cputs( " key'.\r\n    where key is a Drum Key Name (use double quotes if it contains spaces).\r\n");
		_cputs( "    For example, \"Kick 1\" is note 36.\r\n\n");
		_cputs("Version 1.0\r\n");
		exit(0);
	}

	/* Find a match to the supplied Drum Key Name */
	ptr = MidiIsGMDrum(argv[1], &drum, strlen(argv[1]));

	/* Is this a defined Drum Key? If so, then the returned pointer isn't 0 */
	if (!ptr)
	{
		printf("\"%s\" is not one of the defined names.\r\n", argv[1]);
	}

	/* Print the note number (and full Drum Key name in case the user supplied a substring) */
	else
	{
		printf("\"%s\" is note number %u.\r\n", ptr, drum);
	}

	exit(0);
}
